/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.geoviewer.jxmap;

import org.openide.util.Lookup;
import org.openide.util.lookup.ServiceProvider;

/**
 * <p>Class which describes which properties can be set for the Map</p> <p> the
 * different properties which can be set are: Layer list in expanded
 * collapsed</br> Widget actions enabled disabled</br> default layer list node
 * actions enabled disabled</br> </p> 
 * <p>
 * Register your implementation of this class as Service provider of this class via @ServiceProvider(service=MapProperties.class)
 * </p>
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
public abstract class MapProperties {

    private static MapProperties implementation  = Lookup.getDefault().lookup(MapProperties.class);
    
    /**
     * Tries to find the property which determines if the layer list should be
     * expanded or collapsed if no property is set will return false
     *
     * @return
     */
    public static boolean getLayerListExpandedValue() {
        if (implementation != null) {
            return implementation.isLayerListExpanded();
        }
        return false;
    }

    /**
     * Tries to find out if the Layer node remove action should be enabled by
     * looking for {@link ServiceProvider} implementations of this class if no
     * implementation is found returns default true
     *
     * @return
     */
    public static boolean getLayerNodeRemoveActionEnabledValue() {
        if (implementation != null) {
            return implementation.isLayerNodeRemoveActionEnabled();
        }
        return true;
    }

    /**
     * Tries to find out if the Layer node settings action should be enabled by
     * looking for {@link ServiceProvider} implementations of this class if no
     * implementation is found returns default true
     *
     * @return
     */
    public static boolean getLayerNodeSettingsActionEnabledValue() {
        if (implementation != null) {
            return implementation.isLayerNodeSettingsActionEnabled();
        }
        return true;
    }

    /**
     * Tries to find out if the Layer node visibility action should be enabled by
     * looking for {@link ServiceProvider} implementations of this class if no
     * implementation is found returns default true
     *
     * @return
     */
    public static boolean getLayerNodevisibilityActionEnabledValue() {
        if (implementation != null) {
            return implementation.isLayerNodeVisibilityActionEnabled();
        }
        return true;
    }
    
    /**
     * Tries to find out if the actions on the widgets should be enabled by looking for {@link ServiceProvider} implementations of this class if no
     * implementation is found returns default true
     * @return 
     */
    public static boolean getWidgetStateActionsEnabledValue(){
        if (implementation != null) {
            return implementation.widgetStateActionsEnabled();
        }
        return true;
    }
    
    /**
     * Tries to find out if the actions available on the node should be available on the widgets by looking for {@link ServiceProvider} implementations of this class if no
     * implementation is found returns default true
     * @return 
     */
    public static boolean getNodeActionsAvailableOnWidgetValue(){
        if (implementation != null) {
            return implementation.nodeActionsAvailableOnWidget();
        }
        return true;
    } 
    
    /**
     * Should the layer list be expanded
     * @return 
     */
    public abstract boolean isLayerListExpanded();

    /**
     * Should the remove action on the layer node in the layer list be enabled
     * @return 
     */
    public abstract boolean isLayerNodeRemoveActionEnabled();

    /**
     * Should the settings action on the layer node in the layer list be enabled
     * @return 
     */
    public abstract boolean isLayerNodeSettingsActionEnabled();

    /**
     * Should the visibility action on the layer node in the layer list be enabled
     * @return 
     */
    public abstract boolean isLayerNodeVisibilityActionEnabled();
    
    
    /**
     * Should the widget select, hover an object hover actions on a widget be enabled
     * @return 
     */
    public abstract boolean widgetStateActionsEnabled();
    
    /**
     * Should the actions available on the node from which the widget is created be available on the widget itself
     * @return 
     */
    public abstract boolean nodeActionsAvailableOnWidget();
    
    
}
