/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer.jxmap.map;

import java.awt.dnd.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.SingleObjectLayer;
import nl.cloudfarming.client.geoviewer.event.GeoEvent;
import nl.cloudfarming.client.geoviewer.LayerDropTarget;
import org.openide.awt.StatusDisplayer;
import org.openide.nodes.Node;
import org.openide.nodes.NodeTransfer;

/**
 * DropTargetListener implementation which accepts nodes with 
 * layers in their lookup  and delegates the actual dropping to
 * all registered {@link DropTarget}s
 * 
 * 
 * @author Timon Veenstra
 */
public class LayerDropTargetListener implements DropTargetListener {

    private final List<LayerDropTarget> targets = Collections.synchronizedList(new ArrayList<LayerDropTarget>());
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.geoviewer.jxmap.map.LayerDropTargetListener");

    public LayerDropTargetListener() {
    }
    
    public LayerDropTargetListener(LayerDropTarget... droptargets) {
        targets.addAll(Arrays.asList(droptargets));
    }
    
    /**
     * register a new dropTarget which the node dropping gets delegated to
     * 
     * @param dropTarget 
     */
    public void addDropTarget(LayerDropTarget dropTarget){
        targets.add(dropTarget);
    }
    
    public void removeDropTarget(LayerDropTarget dropTarget){
        targets.remove(dropTarget);
    }
   
    @Override
    public void dragEnter(DropTargetDragEvent dtde) {
    }

    @Override
    public void dragExit(DropTargetEvent dtde) {
    }

    @Override
    public void dragOver(DropTargetDragEvent dtde) {
    }

    @Override
    public void dropActionChanged(DropTargetDragEvent dtde) {
    }

    /**
     * Do the actual dropping, drop the node on all registered targets
     * 
     * @param node
     * @return 
     */
    private boolean dropNode(Node node) {
        //TODO verify if we can accept all nodes and just search for 
        // layers. This would allow to drag a root node and add multiple 
        // layers at once. This would also require from refactoring in the 
        // filter nodes
        LOGGER.finest("Incoming dropped node received on the map");
        boolean atLeastOneTargetAccepted = false;
        Layer layer = node.getLookup().lookup(Layer.class);
        if (layer != null) {
            
            StatusDisplayer.getDefault().setStatusText(node.getName());
            for (LayerDropTarget dropTarget:targets){
                boolean accepted = dropTarget.dropNode(node);
                atLeastOneTargetAccepted = atLeastOneTargetAccepted?true:accepted;
            }
        }else{
            LOGGER.log(Level.WARNING, "dropped node {0} did not contain a layer", node.toString());
        }
        return atLeastOneTargetAccepted;
    }

    @Override
    public void drop(DropTargetDropEvent dtde) {
        //
        // handle drop of single nodes
        // (handle single first since multiple also handles single nodes, but not vice versa)
        //
        Node node = NodeTransfer.node(dtde.getTransferable(), NodeTransfer.DND_COPY);
        if (node != null) {
            if (dropNode(node)) {
                dtde.acceptDrop(DnDConstants.ACTION_COPY);
                return;
            }
        }
        //
        // handle drop of multiple nodes
        //
        Node[] nodes = NodeTransfer.nodes(dtde.getTransferable(), NodeTransfer.DND_COPY);
        if (nodes != null) {
//            LOGGER.log(Level.FINEST, "{0} nodes dropped", nodes.length);
            for (Node n : nodes) {
                dropNode(n);
            }
            dtde.acceptDrop(DnDConstants.ACTION_COPY);
            return;
        }
        //
        // unsupported drop, reject
        //
        dtde.rejectDrop();
        dtde.dropComplete(false);
    }
}
