/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.geoviewer.jxmap.map;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Point;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.geoviewer.Geographical;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.LayerInfo;
import nl.cloudfarming.client.geoviewer.event.GeoEvent;
import nl.cloudfarming.client.geoviewer.LayerDropTarget;
import nl.cloudfarming.client.geoviewer.ObjectLayer;
import nl.cloudfarming.client.geoviewer.jxmap.MapViewerTopComponent;
import nl.cloudfarming.client.geoviewer.jxmap.Module;
import nl.cloudfarming.client.geoviewer.jxmap.layerlist.LayerListController;
import nl.cloudfarming.eventbus.GuiEvent;
import nl.cloudfarming.eventbus.GuiEventBus;
import nl.cloudfarming.eventbus.GuiEventKey;
import nl.cloudfarming.eventbus.GuiEventListener;
import org.jdesktop.swingx.mapviewer.GeoPosition;
import org.openide.nodes.Node;

/**
 * Controller of the rootmap panel.
 *
 * The RootMapPanelController is the
 * <code>MapController</code> implementation for the JXMap geoviewer
 * implementation. It exposes methods to control this map implementation.
 * 
* <h3>Swing hierarchy</h3><p> The top swing component for the
 * <code>RootMapController</code> is the {@link MapViewerTopComponent}. This
 * TopComponent only contains the hook to the netbeans window management system,
 * like {@link #setActive(boolean)} on componentActivated and
 * componentDeactivated. The controller delivers the root panel to TopComponent
 * which is a {@link RootMapPanel}. The Controller controls the state and
 * content of this root panel.</p><p> The {@link RootMapPanel} is a
 * <code>JXPanel</code> which contains a JLayerPane. The layered pane contains
 * several layers starting with the JXMap layer as background.</p><p>Layer
 * containing object ({@link ObjectLayer)} implementations) are represented as
 * an {@link ObjectLayerPanel} and added to the layered panel on the
 * {@link RootMapPanel.LayerGroup#LAYER_ACTIVE} position.
 * <code>ObjectLayerPanel</code>s are ObjectScenes and contain
 * <code>LayerObjectWidget</code>s representing geographical components on the
 * map.</p>
 *
 * MapViewerTopComponent->RootMapPanel->ObjectLayerPanel->LayerObjectWidget
 *
 * @author Timon Veenstra
 */
public class RootMapPanelController {
    
    private final RootMapPanel mapPanel;
    private final Map<Layer, LayerPanel> layerPanelMap = new HashMap<>();
    private static int lastId = 0;
    private final LayerDropTargetListener dropTargetListener = new LayerDropTargetListener();
    private final RequestFocusEventListener requestFocusEventListener = new RequestFocusEventListener();
    private static final Logger LOGGER = Logger.getLogger(RootMapPanelController.class.getName());
    
    public RootMapPanelController(LayerListController layerListController) {
        mapPanel = new RootMapPanel(layerListController.getLayerList());
        //
        // register a drop target listener for the layerlist component
        //
        DropTarget dt = new DropTarget(layerListController.getLayerList(), dropTargetListener);
        dt.setDefaultActions(DnDConstants.ACTION_COPY);
        dt.setActive(true);
        layerListController.getLayerList().setDropTarget(dt);
        //
        // register a drop target listener for the top panel of the map component
        //        
        DropTarget dt2 = new DropTarget(mapPanel.getTopPanel(), dropTargetListener);
        dt.setDefaultActions(DnDConstants.ACTION_COPY);
        dt.setActive(true);
        mapPanel.getTopPanel().setDropTarget(dt2);
        //
        // add the request focus listener
        //
        GuiEventBus.addListener(requestFocusEventListener);
    }
    
    public void addLayerDropTarget(LayerDropTarget dropTarget) {
        dropTargetListener.addDropTarget(dropTarget);
    }
    
    public RootMapPanel getPanel() {
        return mapPanel;
    }
    
    public void addLayerNode(Node layerNode, LayerInfo layerInfo) {
        Layer layer = Layer.Finder.findLayer(layerNode);
        //FIXME set propert ID, should depend on layer order since its used for event dispatching        
        LayerPanel layerPanel = new LayerPanel(layerNode, mapPanel.getMapViewer(), lastId++, layerInfo);
        //
        // let layerpanel listen to selection changes in layelist em.
        //
        layer.addPropertyChangeListener(layerPanel);
        //
        //  save reference to the layerpanel and add it to the map panel
        //
        layerPanelMap.put(layer, layerPanel);
        mapPanel.addLayerPanel(layerPanel);
    }
    
    public void removeLayerNode(Node layerNode) {
        final Layer layer = Layer.Finder.findLayer(layerNode);
        if (layer == null) {
            LOGGER.warning("removeLayerNode called with a node without a layer in its hierarchy");
            return;
        }
        LayerPanel layerPanel = layerPanelMap.get(layer);
        //
        // remove reference to the layerpanel and remove it from the map panel
        //
        mapPanel.removeLayerPanel(layerPanel);
        layerPanelMap.remove(layer);
    }
    
    public void refreshLayerNode(Node layerNode) {
        final Layer layer = Layer.Finder.findLayer(layerNode);
        if (layer == null) {
            LOGGER.warning("refreshLayerNode called with a node without a layer in its hierarchy");
            return;
        }
        LayerPanel layerPanel = layerPanelMap.get(layer);
        if (layerPanel != null) {
            layerPanel.repaint();
        } else {
            LOGGER.log(Level.WARNING, "refresh requested for unmanaged layer {0} in node {1}", new Object[]{layer, layerNode});
        }
    }
    
    private class RequestFocusEventListener implements GuiEventListener<Geographical> {
        
        @Override
        public void onEvent(GuiEvent<Geographical> event) {
            Geographical geographical = event.getContent();
            Point centroid = geographical.getCentroid();
            if (centroid != null) {
                GeoPosition geoPosition = new GeoPosition(centroid.getY(), centroid.getX());
                mapPanel.getMapViewer().setCenterPosition(geoPosition);
                //Default zoom level
                mapPanel.getMapViewer().setZoom(3);
                //Now try to zoom out to boundingbox
                if (geographical.getBoundingBox() != null) {
                    Set geoPositions = new HashSet();
                    for (Coordinate coordinate :geographical.getBoundingBox().getCoordinates() ) {
                        GeoPosition position = new GeoPosition(coordinate.y, coordinate.x);
                        geoPositions.add(position);
                    }
                    mapPanel.getMapViewer().calculateZoomFrom(geoPositions);
                } 
            }
        }
        
        @Override
        public boolean listensTo(GuiEventKey guiEventKey) {
            return GeoEvent.REQUEST_FOCUS.equals(guiEventKey);
        }
        
        @Override
        public String getModuleName() {
            return Module.MODULE_NAME;
        }
    }
}
