/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.geoviewer.jxmap.map;

import com.vividsolutions.jts.geom.*;
import com.vividsolutions.jts.geom.impl.CoordinateArraySequence;
import com.vividsolutions.jts.operation.polygonize.Polygonizer;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.swing.AbstractAction;
import nl.cloudfarming.client.geoviewer.SingleObjectLayer;
import nl.cloudfarming.client.geoviewer.SplitGeometryHandler;
import nl.cloudfarming.client.geoviewer.render.DrawingRenderer;
import nl.cloudfarming.client.geoviewer.render.SplitPolygonDrawingRenderer;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import org.jdesktop.swingx.JXMapViewer;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;

/**
 * Split a polygon into two parts based on a drawn line.
 * 
 * @author johan
 */
@NbBundle.Messages({"# {0} - name of object being split", "SplitPolygonDrawingContext.description=splitting geometry for {0}"})
public class SplitPolygonDrawingContext extends AbstractLineDrawingContext {
    private static final AppLogger APP_LOGGER = AppLogFactory.getLogger(SplitPolygonDrawingContext.class);

    private final Node node;
    private final Polygon polygonToSplit;
    private final List<Polygon> results = new ArrayList<>();
    private final SplitGeometryHandler handler;
    
    public SplitPolygonDrawingContext(Node node, JXMapViewer mapViewer) {
        super(node.getLookup().lookup(SingleObjectLayer.class), mapViewer);
        this.handler = node.getLookup().lookup(SplitGeometryHandler.class);
        this.node = node;
        this.polygonToSplit = validateSource(handler.getSource());
    }
    
    /**
     * Try to extract a single polygon from the source geometry.
     * TODO: 
     * - see if this can be combined with the geometry validation from AGROSENSE-824.
     * - create a testable validation class
     * 
     * @param source
     * @return the extracted polygon if successful, throw exception otherwise
     */
    private Polygon validateSource(Geometry source) {
        Polygon polygon;
        
        if (source == null) {
            throw new IllegalArgumentException("source geometry should be not null");
        }
        
        if (source instanceof Polygon) {
            polygon = (Polygon) source;
        }
        else if (source instanceof MultiPolygon) {
            MultiPolygon multiPolygon = (MultiPolygon) source;
            int num = multiPolygon.getNumGeometries();
            if (num != 1) {
                throw new IllegalArgumentException("source multi-polygon should contain exactly one polygon, found: " + num);
            }
            polygon = (Polygon) multiPolygon.getGeometryN(0);
        }
        else {
            throw new IllegalArgumentException("source geometry should be a (multi-)polygon");
        }
        
        if (polygon.isEmpty()) {
            throw new IllegalArgumentException("source polygon should not be empty");
        }
        
        if (polygon.getNumInteriorRing() > 0) {
            APP_LOGGER.warning("Polygon holes aren't handled yet and will be lost when splitting.");
        }
        
        return polygon;
    }

    @Override
    public boolean canFinish() {
        // enough points in splitline?
        if (coords.size() < 2) {
            return false;
        }
        // is the source split into two parts?
        return results.size() == 2;
    }
    
    private void split() {
        results.clear();
        
        // enough points in splitline?
        int len = coords.size();
        if (len < 2) {
            return;
        }

        // create the splitline geometry:
        CoordinateSequence coordSequence = new CoordinateArraySequence(coords.toArray(new Coordinate[len]));
        LineString splitLine = new LineString(coordSequence, geometryFactory);
        
        // since we're splitting into two parts, the intersection should be a single LineString:
        Geometry intersection = splitLine.intersection(polygonToSplit);
        if (intersection == null || intersection.isEmpty() || !(intersection instanceof LineString)) {
            return;
        }
        
        // ensure that the line segments are properly noded and that vertices are created at line intersections:
        // using strategy from http://stackoverflow.com/a/6263275
        // TODO: handle polygon holes
        MultiLineString mls = new MultiLineString(new LineString[] {splitLine, polygonToSplit.getExteriorRing()}, geometryFactory);
        Geometry union = mls.union();
        
        Polygonizer polygonizer = new Polygonizer();
        polygonizer.add(union);
        Collection<Polygon> polygons = polygonizer.getPolygons();

        if (polygons.size() == 2) {
            results.addAll(polygons);
        }

    }

    @Override
    protected void pointAdded() {
        split();
    }
    
    @Override
    protected void pointRemoved() {
        split();
    }

    @Override
    public boolean finish() {
        if (handler.handle(results.toArray(new Geometry[2]), node)) {
            cleanup();
            return true;
        }
        return false;
    }

    @Override
    public DrawingRenderer getRenderer() {
        return new SplitPolygonDrawingRenderer(coords, polygonToSplit, results);
    }

    @Override
    public String getDescription() {
        return Bundle.SplitPolygonDrawingContext_description(node.getDisplayName());
    }
    
    /**
     * Placeholder action for selecting helplines.
     * 
     * TODO: toggle between two mutually exclusive modes: either select a line or draw one.
     */
    @NbBundle.Messages({
            "drawing_select_helpline_action_name=Select helpline",
            "drawing_select_helpline_action_tooltip=Select helpline",
            "drawing_select_helpline_action_icon=nl/cloudfarming/client/geoviewer/jxmap/icons/map16.png"})
    private class SelectHelplineAction extends AbstractAction {

        public SelectHelplineAction() {
            putValue(SHORT_DESCRIPTION, Bundle.drawing_select_helpline_action_tooltip());
            putValue(LARGE_ICON_KEY, ImageUtilities.loadImageIcon(Bundle.drawing_select_helpline_action_icon(), true));
            setEnabled(false);
        }
        
        @Override
        public void actionPerformed(ActionEvent e) {
        }
    }
}
