/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.geoviewer.jxmap.map.cache;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.PriorityBlockingQueue;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.util.connection.ConnectionChecker;
import org.jdesktop.swingx.mapviewer.DefaultTileFactory;
import org.jdesktop.swingx.mapviewer.Tile;
import org.jdesktop.swingx.mapviewer.TileFactoryInfo;
import org.jdesktop.swingx.mapviewer.util.GeoUtil;

/**
 * Copied some code from AbstractTileFactory, so I can use own TileRunner, which does all the
 * reading and writing to the cache
 * 
 * @author Frantisek Post
 */
//TODO we could limit memory cache size, so it uses less memory
public class OfflineTileFactory extends DefaultTileFactory {

    private static final Logger LOG = Logger.getLogger(OfflineTileFactory.class.getName());
    private static BlockingQueue<Tile> tileQueue;
    private ConnectionChecker connectionChecker;
    private Map<String, Tile> tileMap = new HashMap<String, Tile>();
    
    public OfflineTileFactory(TileFactoryInfo tileFactoryInfo) {
        super(tileFactoryInfo);
        connectionChecker = ConnectionChecker.getInstance();
        tileQueue = new PriorityBlockingQueue<Tile>(5,
            new Comparator<Tile>() {
                
                @Override
                public int compare(Tile o1, Tile o2) {
                    if (o1.getPriority() == Tile.Priority.Low && o2.getPriority() == Tile.Priority.High) {
                        return 1;
                    }
                    if (o1.getPriority() == Tile.Priority.High && o2.getPriority() == Tile.Priority.Low) {
                        return -1;
                    }
                    return 0;

                }

                @Override
                public boolean equals(Object obj) {
                    return obj == this;
                }
            });
    }

    @Override
    public Tile getTile(int x, int y, int zoom) {
        return getTileImpl(x, y, zoom, true);
    }

    private Tile getTileImpl(int tpx, int tpy, int zoom, boolean eagerLoad) {
        //wrap the tiles horizontally --> mod the X with the max width
        //and use that
        int tileX = tpx;//tilePoint.getX();
        int numTilesWide = (int) getMapSize(zoom).getWidth();
        if (tileX < 0) {
            tileX = numTilesWide - (Math.abs(tileX) % numTilesWide);
        }

        tileX = tileX % numTilesWide;
        int tileY = tpy < 0 ? 0 : tpy;
        String url = getInfo().getTileUrl(tileX, tileY, zoom);

        Tile tile;
        if (!tileMap.containsKey(url) || needRefresh(url)) { 
            if (!GeoUtil.isValidTile(tileX, tileY, zoom, getInfo())) {
                tile = new Tile(tileX, tileY, zoom);
            } else {
                tile = new OfflineTile(tileX, tileY, zoom, url, this);
                startLoading(tile);
            }
            tileMap.put(url, tile);
        } else {
            tile = tileMap.get(url);
            // if its in the map but is low and isn't loaded yet
            // but we are in high mode
            if (tile.getPriority() == Tile.Priority.Low && eagerLoad && !tile.isLoaded()) {
                promote(tile);
            }
        }

        return tile;
    }

    //we are online, but cached tile contains offline image
    private boolean needRefresh(String url) {
        OfflineTile tile = (OfflineTile) tileMap.get(url);
        return connectionChecker.isOnline() && tile.isOffline();
    }
    

    @Override
    @SuppressWarnings("unchecked")
    public synchronized void startLoading(Tile tile) {
        if (tile.isLoading()) {
            return;
        }
        tile.setLoading(true);
        try {
            tileQueue.put(tile);
            getService().submit(createTileRunner(tile));
        } catch (Exception ex) {
            LOG.log(Level.FINE, "error during load start", ex);
        }
    }

    @Override
    public synchronized void promote(Tile tile) { 
        if (tileQueue.contains(tile)) {
            try {
                tileQueue.remove(tile);
                tile.setPriority(Tile.Priority.High);
                tileQueue.put(tile);
            } catch (Exception ex) {
                LOG.log(Level.FINE, "error while promoting", ex);
            }
        }
    }

    @Override
    protected Runnable createTileRunner(Tile tile) {
        return new OfflineTileRunner(this);
    }

    BlockingQueue<Tile> getTileQueue() {
        return tileQueue;
    }
}
